﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Locomotive
{
    public partial class FormMapWithSetLocomotives : Form
    {
        /// Объект от класса карты с набором объектов
        private MapWithSetLocomotivesGeneric<DrawningObjectLocomotive, AbstractMap> _mapLocomotivesCollectionGeneric;
        public FormMapWithSetLocomotives()
        {
            InitializeComponent();
        }
        /// Выбор карты
        private void comboBoxSelectorMap_SelectedIndexChanged(object sender, EventArgs e)
        {
            AbstractMap map = null;
            switch (comboBoxSelectorMap.Text)
            {
                case "Simple Map":
                    map = new SimpleMap();
                    break;
                case "Spike Map":
                    map = new SpikeMap();
                    break;
                case "Rail Map":
                    map = new RailroadMap();
                    break;
            }
            if (map != null)
            {
                _mapLocomotivesCollectionGeneric = new MapWithSetLocomotivesGeneric<DrawningObjectLocomotive, AbstractMap>
                    (pictureBox.Width, pictureBox.Height, map);
            }
            else
            {
                _mapLocomotivesCollectionGeneric = null;
            }
        }
        /// Добавление объекта
        private void buttonAddLocomotive_Click(object sender, EventArgs e)
        {
            if (_mapLocomotivesCollectionGeneric == null)
            {
                return;
            }
            FormLocomotive form = new();
            if (form.ShowDialog() == DialogResult.OK)
            {
                DrawningObjectLocomotive locomotive = new(form.SelectedLocomotive);
                if (_mapLocomotivesCollectionGeneric + locomotive != -1)
                {
                    MessageBox.Show("Объект добавлен");
                    pictureBox.Image = _mapLocomotivesCollectionGeneric.ShowSet();
                }
                else
                {
                    MessageBox.Show("Не удалось добавить объект");
                }
            }
        }
        /// Удаление объекта
        private void buttonRemoveLocomotive_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(maskedTextBoxPosition.Text))
            {
                return;
            }
            if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
            if (_mapLocomotivesCollectionGeneric - pos is not null)
            {
                MessageBox.Show("Объект удален");
                pictureBox.Image = _mapLocomotivesCollectionGeneric.ShowSet();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }
        /// Вывод набора
        private void buttonShowStorage_Click(object sender, EventArgs e)
        {
            if (_mapLocomotivesCollectionGeneric == null)
            {
                return;
            }
            pictureBox.Image = _mapLocomotivesCollectionGeneric.ShowSet();
        }
        /// Вывод карты
        private void buttonShowOnMap_Click(object sender, EventArgs e)
        {
            if (_mapLocomotivesCollectionGeneric == null)
            {
                return;
            }
            pictureBox.Image = _mapLocomotivesCollectionGeneric.ShowOnMap();
        }
        /// Перемещение
        private void buttonMove_Click(object sender, EventArgs e)
        {
            if (_mapLocomotivesCollectionGeneric == null)
            {
                return;
            }
            //получаем имя кнопки
            string name = ((Button)sender)?.Name ?? string.Empty;
            Direction dir = Direction.None;
            switch (name)
            {
                case "buttonUp":
                    dir = Direction.Up;
                    break;
                case "buttonDown":
                    dir = Direction.Down;
                    break;
                case "buttonLeft":
                    dir = Direction.Left;
                    break;
                case "buttonRight":
                    dir = Direction.Right;
                    break;
            }
            pictureBox.Image = _mapLocomotivesCollectionGeneric.MoveObject(dir);
        }
    }
}
